/*
Written by Ed Hong
UWT
May 26, 2003
(based on previous PQ code also written by Ed Hong.)

 */

import java.util.*;


public class SortedArrayPQ {

    // internal class for containing 
    // pq items, including (key,object,and locator). 
    // no methods beside constructor. 
  private class pqitem {
      public double key;
      public Object element;
      public sapqLoc loc;
      public pqitem(double k, Object o) {
	key = k;
	element = o;	
	loc = new sapqLoc(-1); // initialize locator to point nowhere.
      }
  }

  private class sapqLoc implements Locator {
      private int locind; // index to location of item inside pq
      // constructor.. 
      private sapqLoc(int i) {
	  locind = i;
      }
      public double key() {
	  /* returns key (priority) associated to referenced data */
	  return ((pqitem)pqdata.elementAt(locind)).key;
      }
      public Object element() {
	  /* returns Object associated to referenced data */
	  return ((pqitem) pqdata.elementAt(locind)).element;
      }
  }

    private Vector pqdata; // stores maximum on left, minimum on right

    public SortedArrayPQ() {
	pqdata = new Vector();
    }

    // inserts k into pqdata at the appropriate place. 
    private void inorderInsert(pqitem k) {
	pqitem curr = null;
	int ind;
	if (pqdata.size()==0) { 
	    k.loc.locind = 0;
	    pqdata.add(k);
	    return;
	}
	for ( ind=0; ind < pqdata.size(); ind++) {
	    curr = (pqitem) pqdata.get(ind);
	    /*	    System.out.println("PQDAT: item " + curr + " (" + ind + ") is (" + 
			       curr.key + ", " + 
			       curr.element + ", " + 
			       curr.loc + " " + 
			       curr.loc.locind + 
			       ")");*/
	    if (curr.key < k.key) {
		// break out of loop and insert here
		break;
	    }
	}
	pqitem temp = k;
	int i;
	// now insert item, shifting things to right, 
	// updating locators along the way. 
	// invariant: temp is next item to be put at index i
	//            curr contains item currently at index i that will
        //            be replaced
	for (i=ind; i+1 < pqdata.size(); i++) {
	    temp.loc.locind = i;
	    pqdata.setElementAt(temp,i);
	    /*	    System.out.println("PQUPDAT: item " + temp + " (" + i + ") is (" + 
			       temp.key + ", " + 
			       temp.element + ", " + 
			       temp.loc + " " + temp.loc.locind + ")");
	    */
	    temp = curr;
	    curr = (pqitem) pqdata.get(i+1);
	}
	temp.loc.locind = i;
	if (i >= pqdata.size()) {
	    pqdata.addElement(temp);
	    return;
	}
	pqdata.setElementAt(temp,i);
	curr.loc.locind = i+1;
	pqdata.addElement(curr); // add final element in. 
    }

    public Locator insertItem(double key, Object o) {
	pqitem newItem;
	newItem = new pqitem(key, o); 
	inorderInsert(newItem);
	return newItem.loc;
    }

    public Object removeMin() {
	pqitem temp; 

	if (pqdata.size()==0) return null;
	temp = (pqitem) pqdata.lastElement();
	pqdata.removeElementAt(pqdata.size()-1);

	temp.loc.locind = -1;
	/*	System.out.println("RemoveMin: item " + temp + " is (" + 
			       temp.key + ", " + 
			       temp.element + ", " + 
			       temp.loc + " " + temp.loc.locind + ")");
	*/
	return temp.element;
    }
    
    public void remove(Locator l) {
	pqitem temp;
	// remove item, shifting things to left, 
	// updating locators along the way. 
	// invariant: temp is next item to be put at index i
	//            curr contains item currently at index i that will
        //            be replaced
	sapqLoc sl = (sapqLoc) l;
	if (sl.locind==-1) {
	    System.out.println("Error in remove(): locator refers to object not in PQ! Ignoring call!");
	    return;
	}
	int i;
	for (i=sl.locind; i+1 < pqdata.size(); i++) {
	    temp = (pqitem) pqdata.get(i+1);
	    temp.loc.locind = i;
	    pqdata.setElementAt(temp,i);
	}
	sl.locind = -1;
	pqdata.removeElementAt(i);
    }

    public void replaceKey(Locator l, double newkey) {
	sapqLoc sl = (sapqLoc) l;
	if (sl.locind==-1) {
	    System.out.println("Error in replaceKey(): locator refers to object not in PQ! Ignoring call!");
	    return;
	}
	/*	System.out.println("ReplaceKey:  newkey " + newkey + 
		"; loc = " + l + " locindt = " + sl.locind + ";");*/
	pqitem temp = (pqitem) pqdata.get(sl.locind); 
	remove(l);
	temp.key = newkey;
	inorderInsert(temp);
    }

    public boolean isEmpty() {
	return (pqdata.size() == 0);
    }
}




