/*
 * Written by Ed Hong UWT Feb. 19, 2003.
 * Modified by Donald Chinn May 14, 2003.
 */

import java.util.*;

/**
 * Class that represents a vertex in a graph.
 * An object (usually some sort of data) can be associated with the vertex.
 * 
 * A label (also represented by an object (e.g., a string) can also be
 * associated with a vertex.  This could be useful, for example, if you
 * need to mark a vertex as being visited in some graph traversal.
 * 
 * @author edhong
 * @version 0.0
 */
public class Vertex {
    /** the edge list for this vertex */
    LinkedList incidentEdgeList;

    private Object elementdat;              // an object associated with this vertex
    private Object visitLabel;              // a label associated with this vertex
    
    /**
     * Constructor that allows an object to be associated
     * with the vertex.
     * @param o     an object to be associated with this vertex
     */
    public Vertex(Object o) {
        this.elementdat = o;
        this.incidentEdgeList = new LinkedList();
    }
    
    /**
     * Return the object associated with this vertex.
     * @return  the object of this vertex
     */
    public Object element(){
        return this.elementdat;
    }
    
    /**
     * Return the label associated with this vertex.
     * @return  the label of this vertex
     */
    public Object getLabel() {
        return visitLabel;
    }
    
    /**
     * Set the label associated with this vertex.
     * @param o  the label of this vertex
     */
    public void setLabel(Object o) {
        visitLabel = o;
    }
}
